package com.jiuqi.sql.impl;

import java.util.Date;
import com.jiuqi.sql.FieldInfo;
import java.io.IOException;
import com.jiuqi.sql.DBException;
import java.io.InputStream;
import com.jiuqi.sql.InputRecordBuffer;

public class DBaseInputRecordBuffer implements InputRecordBuffer
{
    private char deleteFlag;
    private byte[] buffer;
    private boolean prevNull;
    private DBaseFileTail strValueList;
    private boolean supportText;
    
    public DBaseInputRecordBuffer(final int recordSize, final boolean supportText, final DBaseFileTail strValueList) {
        this.buffer = new byte[recordSize];
        this.supportText = supportText;
        this.strValueList = strValueList;
    }
    
    public boolean readRecord(final InputStream stream) throws DBException {
        boolean result = false;
        try {
            this.deleteFlag = (char)stream.read();
            if (!this.supportText || this.deleteFlag != '\u001a') {
                final int readSize = stream.read(this.buffer);
                result = (readSize == this.buffer.length);
            }
        }
        catch (final IOException ex) {
            final DBException dbe = new DBException();
            dbe.setStackTrace(ex.getStackTrace());
            throw dbe;
        }
        return result;
    }
    
    public boolean isDeleted() {
        return this.deleteFlag == '*';
    }
    
    private boolean isNull(final FieldInfo info) {
        switch (info.type) {
            case 1: {
                return this.buffer[info.offset] == 32;
            }
            case 3: {
                boolean flag = false;
                for (int i = 0; i < 8; ++i) {
                    if (this.buffer[info.offset + i] != 0) {
                        flag = true;
                        break;
                    }
                }
                return !flag;
            }
            case 5: {
                boolean flag = false;
                for (int i = 0; i < 4; ++i) {
                    if (this.buffer[info.offset + i] != 0) {
                        flag = true;
                        break;
                    }
                }
                return !flag;
            }
            case 6: {
                boolean flag = false;
                int empty = 0;
                int size = info.size;
                if (this.supportText) {
                    empty = -1;
                    size = 4;
                }
                for (int j = 0; j < size; ++j) {
                    if (this.buffer[info.offset + j] != empty) {
                        flag = true;
                        break;
                    }
                }
                return !flag;
            }
            case 2:
            case 8: {
                boolean flag = false;
                for (int i = 0; i < 8; ++i) {
                    if (this.buffer[info.offset + i] != 32) {
                        flag = true;
                        break;
                    }
                }
                return !flag;
            }
            default: {
                return false;
            }
        }
    }
    
    public boolean readBoolean(final FieldInfo info) throws DBException {
        this.prevNull = this.isNull(info);
        return !this.prevNull && RW.readBoolean(this.buffer, info.offset);
    }
    
    public double readDouble(final FieldInfo info) throws DBException {
        this.prevNull = this.isNull(info);
        return this.prevNull ? 0.0 : RW.readDouble(this.buffer, info.offset);
    }
    
    public int readInt(final FieldInfo info) throws DBException {
        this.prevNull = this.isNull(info);
        return this.prevNull ? 0 : RW.readInt(this.buffer, info.offset);
    }
    
    public String readString(final FieldInfo info) throws DBException {
        this.prevNull = this.isNull(info);
        if (this.prevNull) {
            return null;
        }
        if (this.supportText) {
            final int index = RW.readInt(this.buffer, info.offset);
            return this.strValueList.get(index);
        }
        return RW.readString(this.buffer, info.offset, info.size);
    }
    
    public long readLong(final FieldInfo info) throws DBException {
        this.prevNull = this.isNull(info);
        return this.prevNull ? 0L : RW.readLong(this.buffer, info.offset);
    }
    
    public Date readDate(final FieldInfo info) throws DBException {
        this.prevNull = this.isNull(info);
        return this.prevNull ? null : RW.readDateTime(this.buffer, info.offset);
    }
    
    public boolean wasNull() throws DBException {
        return this.prevNull;
    }
}
